@@ -20,13 +20,13 @@ require CPANPLUS::Internals;
 
 =head1 NAME
 
-CPANPLUS::Internals::Report
+CPANPLUS::Internals::Report - internals for sending test reports
 
 =head1 SYNOPSIS
 
   ### enable test reporting
   $cb->configure_object->set_conf( cpantest => 1 );
-    
+
   ### set custom mx host, shouldn't normally be needed
   $cb->configure_object->set_conf( cpantest_mx => 'smtp.example.com' );
 
@@ -104,7 +104,7 @@ otherwise.
 
 This function queries the CPAN testers database at
 I<http://testers.cpan.org/> for test results of specified module objects,
-module names or distributions. 
+module names or distributions.
 
 The optional argument C<all_versions> controls whether all versions of
 a given distribution should be grabbed.  It defaults to false
@@ -185,7 +185,7 @@ sub _query_report {
     my $res = do {
         my $tempdir = File::Temp::tempdir();
         my $where   = $ff->fetch( to => $tempdir );
-        
+
         unless( $where ) {
             error( loc( "Fetching report for '%1' failed: %2",
                         $url, $ff->error ) );
@@ -193,7 +193,7 @@ sub _query_report {
         }
 
         my $fh = OPEN_FILE->( $where );
-        
+
         do { local $/; <$fh> };
     };
 
@@ -209,11 +209,11 @@ sub _query_report {
 
     my @rv;
     for my $href ( @$aref ) {
-        next unless $all or defined $href->{'distversion'} && 
+        next unless $all or defined $href->{'distversion'} &&
                             $href->{'distversion'} eq $dist;
 
         $href->{'details'}  = $details;
-        
+
         ### backwards compatibility :(
         $href->{'dist'}     ||= $href->{'distversion'};
         $href->{'grade'}    ||= $href->{'action'} || $href->{'status'};
@@ -296,19 +296,20 @@ sub _send_report {
     }
 
     ### check arguments ###
-    my ($buffer, $failed, $mod, $verbose, $force, $address, $save, 
-        $tests_skipped );
+    my ($buffer, $failed, $mod, $verbose, $force, $address, $save,
+        $tests_skipped, $status );
     my $tmpl = {
             module  => { required => 1, store => \$mod, allow => IS_MODOBJ },
             buffer  => { required => 1, store => \$buffer },
             failed  => { required => 1, store => \$failed },
+            status  => { default => {}, store => \$status, strict_type => 1 },
             address => { default  => CPAN_TESTERS_EMAIL, store => \$address },
             save    => { default  => 0, store => \$save },
             verbose => { default  => $conf->get_conf('verbose'),
                             store => \$verbose },
             force   => { default  => $conf->get_conf('force'),
                             store => \$force },
-            tests_skipped   
+            tests_skipped
                     => { default => 0, store => \$tests_skipped },
     };
 
@@ -332,22 +333,22 @@ sub _send_report {
 
     my $grade;
     ### check if this is a platform specific module ###
-    ### if we failed the test, there may be reasons why 
+    ### if we failed the test, there may be reasons why
     ### an 'NA' might have to be instead
     GRADE: { if ( $failed ) {
-        
+
 
         ### XXX duplicated logic between this block
         ### and REPORTED_LOADED_PREREQS :(
-        
+
         ### figure out if the prereqs are on CPAN at all
         ### -- if not, send NA grade
         ### Also, if our version of prereqs is too low,
         ### -- send NA grade.
-        ### This is to address bug: #25327: do not count 
+        ### This is to address bug: #25327: do not count
         ### as FAIL modules where prereqs are not filled
         {   my $prq = $mod->status->prereqs || {};
-        
+
             PREREQ: while( my($prq_name,$prq_ver) = each %$prq ) {
 
                 # 'perl' listed as prereq
@@ -362,12 +363,13 @@ sub _send_report {
                       $grade = GRADE_NA;
                       last GRADE;
                    }
+                   next PREREQ;
                 }
 
                 my $obj = $cb->module_tree( $prq_name );
-                my $sub = CPANPLUS::Module->can(         
+                my $sub = CPANPLUS::Module->can(
                             'module_is_supplied_with_perl_core' );
-                
+
                 ### if we can't find the module and it's not supplied with core.
                 ### this addresses: #32064: NA reports generated for failing
                 ### tests where core prereqs are specified
@@ -377,7 +379,7 @@ sub _send_report {
                 ###    http://rt.cpan.org/Ticket/Display.html?id=32155
                 if( !$obj and !defined $sub->( $prq_name ) ) {
                     msg(loc( "Prerequisite '%1' for '%2' could not be obtained".
-                             " from CPAN -- sending N/A grade", 
+                             " from CPAN -- sending N/A grade",
                              $prq_name, $name ), $verbose );
 
                     $grade = GRADE_NA;
@@ -388,10 +390,10 @@ sub _send_report {
                     my $vcore = $sub->( $prq_name );
                     if ( $cb->_vcmp( $prq_ver, $vcore ) > 0 ) {
                       msg(loc( "Version of core module '%1' ('%2') is too low for ".
-                               "'%3' (needs '%4') -- sending N/A grade", 
-                               $prq_name, $vcore, 
+                               "'%3' (needs '%4') -- sending N/A grade",
+                               $prq_name, $vcore,
                                $name, $prq_ver ), $verbose );
-                             
+
                       $grade = GRADE_NA;
                       last GRADE;
                     }
@@ -399,33 +401,33 @@ sub _send_report {
 
                 if( $obj and $cb->_vcmp( $prq_ver, $obj->installed_version ) > 0 ) {
                     msg(loc( "Installed version of '%1' ('%2') is too low for ".
-                             "'%3' (needs '%4') -- sending N/A grade", 
-                             $prq_name, $obj->installed_version, 
+                             "'%3' (needs '%4') -- sending N/A grade",
+                             $prq_name, $obj->installed_version,
                              $name, $prq_ver ), $verbose );
-                             
+
                     $grade = GRADE_NA;
                     last GRADE;
-                }                             
+                }
             }
         }
-        
+
         unless( RELEVANT_TEST_RESULT->($mod) ) {
             msg(loc(
                 "'%1' is a platform specific module, and the test results on".
                 " your platform are not relevant --sending N/A grade.",
                 $name), $verbose);
-        
+
             $grade = GRADE_NA;
-        
+
         } elsif ( UNSUPPORTED_OS->( $buffer ) ) {
             msg(loc(
                 "'%1' is a platform specific module, and the test results on".
                 " your platform are not relevant --sending N/A grade.",
                 $name), $verbose);
-        
+
             $grade = GRADE_NA;
-        
-        ### you dont have a high enough perl version?    
+
+        ### you dont have a high enough perl version?
         } elsif ( PERL_VERSION_TOO_LOW->( $buffer ) ) {
             msg(loc("'%1' requires a higher version of perl than your current ".
                     "version -- sending N/A grade.", $name), $verbose);
@@ -470,8 +472,9 @@ sub _send_report {
         return 1 if $cp_conf =~  /\bmaketest_only\b/i
                     and ($stage !~ /\btest\b/);
 
+        my $capture = ( $status && defined $status->{capture} ? $status->{capture} : $buffer );
         ### the bit where we inform what went wrong
-        $message .= REPORT_MESSAGE_FAIL_HEADER->( $stage, $buffer );
+        $message .= REPORT_MESSAGE_FAIL_HEADER->( $stage, $capture );
 
         ### was it missing prereqs? ###
         if( my @missing = MISSING_PREREQS_LIST->($buffer) ) {
@@ -505,9 +508,34 @@ sub _send_report {
     } elsif ( $tests_skipped ) {
         $message .= REPORT_TESTS_SKIPPED->();
     } elsif( $grade eq GRADE_NA) {
-    
+
+        ### add the reason for the NA to the buffer
+        $buffer = join $/, $buffer, map {
+                        '[' . $_->tag . '] [' . $_->when . '] ' .
+                        $_->message } ( CPANPLUS::Error->stack )[-1];
+
+        my $capture = ( $status && defined $status->{capture} ? $status->{capture} : $buffer );
         ### the bit where we inform what went wrong
-        $message .= REPORT_MESSAGE_FAIL_HEADER->( $stage, $buffer );
+        $message .= REPORT_MESSAGE_FAIL_HEADER->( $stage, $capture );
+
+        ### add a list of what modules have been loaded of your prereqs list
+        $message .= REPORT_LOADED_PREREQS->($mod);
+
+        ### add a list of versions of toolchain modules
+        $message .= REPORT_TOOLCHAIN_VERSIONS->($mod);
+
+        ### the footer
+        $message .= REPORT_MESSAGE_FOOTER->();
+
+    } elsif ( $grade eq GRADE_PASS and ( $status and defined $status->{capture} ) ) {
+        ### the bit where we inform what went right
+        $message .= REPORT_MESSAGE_PASS_HEADER->( $stage, $status->{capture} );
+
+        ### add a list of what modules have been loaded of your prereqs list
+        $message .= REPORT_LOADED_PREREQS->($mod);
+
+        ### add a list of versions of toolchain modules
+        $message .= REPORT_TOOLCHAIN_VERSIONS->($mod);
 
         ### the footer
         $message .= REPORT_MESSAGE_FOOTER->();
@@ -519,13 +547,13 @@ sub _send_report {
     ### reporter object ###
     my $reporter = do {
         my $args = $conf->get_conf('cpantest_reporter_args') || {};
-        
+
         unless( UNIVERSAL::isa( $args, 'HASH' ) ) {
             error(loc("'%1' must be a hashref, ignoring...",
                       'cpantest_reporter_args'));
             $args = {};
         }
-        
+
         Test::Reporter->new(
             grade           => $grade,
             distribution    => $dist,
@@ -536,9 +564,9 @@ sub _send_report {
             %$args,
         );
     };
-    
+
     ### set a custom mx, if requested
-    $reporter->mx( [ $conf->get_conf('cpantest_mx') ] ) 
+    $reporter->mx( [ $conf->get_conf('cpantest_mx') ] )
         if $conf->get_conf('cpantest_mx');
 
     ### set the from address ###
@@ -585,10 +613,10 @@ sub _send_report {
 
     ### XXX should we do an 'already sent' check? ###
     ### something broke :( ###
-    } 
+    }
     else {
         my $status;
-        eval { 
+        eval {
             $status = $reporter->send();
         };
         if ( $@ ) {
@@ -620,7 +648,7 @@ sub _verify_missing_prereqs {
 
     check( $tmpl, \%hash ) or return;
 
-    
+
     my %missing = map {$_ => 1} @$missing;
     my $conf = $self->configure_object;
     my $extract = $mod->status->extract;
@@ -640,9 +668,9 @@ sub _verify_missing_prereqs {
     for my $file ( @search ) {
         if(-e $file and -r $file) {
             my $slurp = $self->_get_file_contents(file => $file);
-            my ($prereq) = 
+            my ($prereq) =
                 ($slurp =~ /'?(?:PREREQ_PM|requires)'?\s*=>\s*{(.*?)}/s);
-            my @prereq = 
+            my @prereq =
                 ($prereq =~ /'?([\w\:]+)'?\s*=>\s*'?\d[\d\.\-\_]*'?/sg);
             delete $missing{$_} for(@prereq);
         }
